use super::Solution;

pub struct Day14 {}

impl Solution for Day14 {
    fn part1(
        &self,
        input: &mut Vec<String>,
    ) -> Result<Box<dyn std::fmt::Display + Sync>, Box<dyn std::error::Error>> {
        let mut rock_formation = input
            .iter()
            .map(|r| r.chars().collect::<Vec<_>>())
            .collect::<Vec<_>>();

        self.pull_north(&mut rock_formation);

        Ok(Box::new(self.get_load(&rock_formation)))
    }

    fn part2(
        &self,
        _input: &mut Vec<String>,
    ) -> Result<Box<dyn std::fmt::Display + Sync>, Box<dyn std::error::Error>> {
        Ok(Box::new("Ready"))
    }

    fn get_day(&self) -> u8 {
        14
    }
}

impl Day14 {
    // Function that pulls all of the 'O's as far north as they will go
    fn pull_north(&self, cave: &mut Vec<Vec<char>>) {
        let mut complete = false;

        while !complete {
            complete = true;
            for row in 1..cave.len() {
                for col in 0..cave[0].len() {
                    if cave[row][col] == 'O' && cave[row - 1][col] == '.' {
                        cave[row][col] = '.';
                        cave[row - 1][col] = 'O';
                        complete = false;
                    }
                }
            }
        }
    }

    // Function that calculates the load on the north beams of the cave
    fn get_load(&self, cave: &Vec<Vec<char>>) -> u32 {
        (0..cave.len())
            .map(|row| {
                (0..cave[0].len())
                    .filter(|col| cave[row][*col] == 'O')
                    .map(|_| cave.len() - row)
                    .sum::<usize>()
            })
            .sum::<usize>() as u32
    }
}

/// Test from puzzle input
#[cfg(test)]
mod test {
    use super::*;
    use crate::*;

    #[test]
    fn part1() {
        let challenge = day14::Day14 {};

        //Complete the Challenge
        let answer = challenge
            .part1(
                utils::get_input(challenge.get_day(), utils::InputType::Test1)
                    .unwrap()
                    .as_mut(),
            )
            .unwrap()
            .to_string();

        assert_eq!(answer, "136");
    }

    #[test]
    fn part2() {
        let challenge = day14::Day14 {};

        //Complete the Challenge
        let answer = challenge
            .part2(
                utils::get_input(challenge.get_day(), utils::InputType::Test2)
                    .unwrap()
                    .as_mut(),
            )
            .unwrap()
            .to_string();

        assert_eq!(answer, "Ready");
    }
}
